# Upgrade steps from eZ Publish 5.2 to 5.3

## Session name
`ezpublish.system.<siteAccessName>.session_name` has been deprecated for defining session name.
You now need to use `ezpublish.system.<siteAccessName>.session.name`.

*Before*:
```yaml
ezpublish:
    system:
        my_siteaccess:
            session_name: SomeSessionName
```

*After*:
```yaml
ezpublish:
    system:
        my_siteaccess:
            session:
                name: SomeSessionName
```

## Database configuration
`ezpublish.system.<siteAccessName>.database` has been removed for defining database settings.
You now need to:
* Define a Doctrine connection:

  *ezpublish.yml* or *config.yml*
  ```yaml
  doctrine:
      dbal:
          connections:
              my_connection:
                  driver:   pdo_mysql
                  host:     localhost
                  port:     3306
                  dbname:   my_database
                  user:     my_user
                  password: my_password
                  charset:  UTF8
  ```
  > **Pro tip:** Set your base DB params in your `parameters.yml`/`parameters.yml.dist` and refer them here.

   *parameters.yml*
   ```yaml
   parameters:
       database_driver: pdo_mysql
       database_host: localhost
       database_port: 3306
       database_name: ezdemo
       database_user: my_user
       database_password: my_password
       database_charset: UTF8
   ```

   *ezpublish.yml / config.yml*
   ```yaml
   doctrine:
       dbal:
           connections:
               my_connection:
                   driver:   %database_driver%
                   host:     %database_host%
                   port:     %database_port
                   dbname:   %database_name%
                   user:     %database_user
                   password: %database_password
                   charset:  %database_charset%
   ```

* Define one or several repositories:

  *ezpublish.yml*
  ```yaml
  ezpublish:
    repositories:
        main: { engine: legacy, connection: my_connection }
  ```

* (Optional) Make your SiteAccess config point to the right repository:

  *ezpublish.yml*
  ```yaml
  ezpublish:
    system:
        my_siteaccess_group:
            repository: main
  ```
